

# BinaryMatcher2
### D. Thiebaut
### March 19, 2017

This Jupyter Notebook illustrates how to design a simple multi-layer Tensorflow Neural Net to recognize integers coded in binary and output them as 1-hot vector.

For example, if we assume that we have 5 bits, then there are 32 possible combinations.  We associate with each 5-bit sequence a 1-hot vector.  For example, 0,0,0,1,1, which is 3 in decimal, is associated with 0,0,0,1,0,0,0,0...,0, which has 31 0s and one 1.  The only 1 is at Index 3.  Similarly, if we have 1,1,1,1,1, which is 31 in decimal, then its associated 1-hot vector is 0,0,0,0,...0,0,1, another group of 31 0s and one last 1.

Our binary input is coded in 5 bits, and we make it more interesting by adding 5 additional random bits.  So the input is a vector of 10 bits, 5 random, and 5 representing a binary pattern associated with a 1-hot vector.  The 1-hot vector is the output to be predicted by the network.


## Preparing the Data

Let's prepare a set of data where we have 5 bits of input, plus 3 random bits, plus 32 outputs corresponding to 1-of for the integer coded in the 5 bits.

## Preparing the Raw Data: 32 rows Binary and 1-Hot
We first create two arrays of 32 rows.  The first array, called x32, contains the binary patterns for 0 to 31.  The second array, called y32, contains the one-hot version of the equivalent entry in the x32 array.  For example, [0,0,0,0,0] in x32 corresponds to [1,0,0,0,...,0] (one 1 followed by thirty one 0s) in y32.  [1,1,1,1,1] in x32 corresponds to [0,0,0...,0,0,1] in y32.


```python
from __future__ import print_function
import random
import numpy as np
import tensorflow as tf

# create the 32 binary values of 0 to 31
# as well as the 1-hot vector of 31 0s and one 1.
x32 = []
y32 = []
for i in range( 32 ):
    n5 = ("00000" + "{0:b}".format(i))[-5:]
    bits = [0]*32
    bits[i] = 1
    #print( n5,":", r3, "=", bits )
    nBits = [ int(n) for n in n5 ]
   
    #print( nBits, rBits, bits )
    #print( type(x), type(y), type(nBits), type(rBits), type( bits ))
    x32 = x32 +  [nBits]
    y32 = y32 +  [bits] 
    
# print both collections to verify that we have the correct data.
# The x vectors will be fed to the neural net (NN) (along with some noisy data), and
# we'll train the NN to generate the correct 1-hot vector.
print( "x = ", "\n".join( [str(k) for k in x32] ) )
print( "y = ", "\n".join( [str(k) for k in y32] ) )
    
```

    x =  [0, 0, 0, 0, 0]
    [0, 0, 0, 0, 1]
    [0, 0, 0, 1, 0]
    [0, 0, 0, 1, 1]
    [0, 0, 1, 0, 0]
    [0, 0, 1, 0, 1]
    [0, 0, 1, 1, 0]
    [0, 0, 1, 1, 1]
    [0, 1, 0, 0, 0]
    [0, 1, 0, 0, 1]
    [0, 1, 0, 1, 0]
    [0, 1, 0, 1, 1]
    [0, 1, 1, 0, 0]
    [0, 1, 1, 0, 1]
    [0, 1, 1, 1, 0]
    [0, 1, 1, 1, 1]
    [1, 0, 0, 0, 0]
    [1, 0, 0, 0, 1]
    [1, 0, 0, 1, 0]
    [1, 0, 0, 1, 1]
    [1, 0, 1, 0, 0]
    [1, 0, 1, 0, 1]
    [1, 0, 1, 1, 0]
    [1, 0, 1, 1, 1]
    [1, 1, 0, 0, 0]
    [1, 1, 0, 0, 1]
    [1, 1, 0, 1, 0]
    [1, 1, 0, 1, 1]
    [1, 1, 1, 0, 0]
    [1, 1, 1, 0, 1]
    [1, 1, 1, 1, 0]
    [1, 1, 1, 1, 1]
    y =  [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]



## Addition of Random Bits
Let's add some random bits (say 7) to the rows of x, and create a larger collection of rows, say 100. 



```python
x = []
y = []
noRandomBits = 5
for i in range( 100 ):
    # pick all the rows in a round-robin fashion.
    xrow = x32[i%32]
    yrow = y32[i%32]
    
    # generate a random int of 5 bits
    r5 = random.randint( 0, 31 )
    r5 = ("0"*noRandomBits + "{0:b}".format(r5))[-noRandomBits:]

    # create a list of integer bits for r5
    rBits = [ int(n) for n in r5 ]
    
    #create a new row of x and y values
    x.append( xrow + rBits )
    y.append( yrow )
    
# display x and y
for i in range( len( x ) ):
    print( "x[%2d] ="%i, ",".join( [str(k) for k in x[i] ] ), "y[%2d] ="%i, ",".join( [str(k) for k in y[i] ] ) )

```

    x[ 0] = 0,0,0,0,0,1,0,0,0,1 y[ 0] = 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[ 1] = 0,0,0,0,1,1,1,0,1,1 y[ 1] = 0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[ 2] = 0,0,0,1,0,0,0,1,0,0 y[ 2] = 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[ 3] = 0,0,0,1,1,1,1,1,0,0 y[ 3] = 0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[ 4] = 0,0,1,0,0,0,0,1,1,1 y[ 4] = 0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[ 5] = 0,0,1,0,1,0,1,1,1,0 y[ 5] = 0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[ 6] = 0,0,1,1,0,1,1,0,1,1 y[ 6] = 0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[ 7] = 0,0,1,1,1,0,0,1,1,0 y[ 7] = 0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[ 8] = 0,1,0,0,0,0,0,1,0,0 y[ 8] = 0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[ 9] = 0,1,0,0,1,0,0,0,1,1 y[ 9] = 0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[10] = 0,1,0,1,0,0,0,0,1,1 y[10] = 0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[11] = 0,1,0,1,1,0,1,0,1,0 y[11] = 0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[12] = 0,1,1,0,0,1,0,1,1,0 y[12] = 0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[13] = 0,1,1,0,1,1,0,1,0,0 y[13] = 0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[14] = 0,1,1,1,0,0,1,0,1,0 y[14] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[15] = 0,1,1,1,1,1,1,1,0,1 y[15] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[16] = 1,0,0,0,0,0,1,1,0,1 y[16] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[17] = 1,0,0,0,1,1,1,1,0,1 y[17] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[18] = 1,0,0,1,0,0,0,1,0,1 y[18] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[19] = 1,0,0,1,1,1,0,0,0,1 y[19] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0
    x[20] = 1,0,1,0,0,1,0,1,1,0 y[20] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0
    x[21] = 1,0,1,0,1,1,1,0,1,1 y[21] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0
    x[22] = 1,0,1,1,0,1,0,1,0,0 y[22] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0
    x[23] = 1,0,1,1,1,1,0,0,1,1 y[23] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0
    x[24] = 1,1,0,0,0,1,0,0,1,0 y[24] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0
    x[25] = 1,1,0,0,1,1,1,0,0,0 y[25] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0
    x[26] = 1,1,0,1,0,0,1,1,1,0 y[26] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0
    x[27] = 1,1,0,1,1,0,1,0,0,1 y[27] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0
    x[28] = 1,1,1,0,0,1,0,1,0,0 y[28] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0
    x[29] = 1,1,1,0,1,0,0,1,1,1 y[29] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
    x[30] = 1,1,1,1,0,1,0,1,0,0 y[30] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0
    x[31] = 1,1,1,1,1,1,0,1,1,0 y[31] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
    x[32] = 0,0,0,0,0,0,0,0,1,1 y[32] = 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[33] = 0,0,0,0,1,0,1,1,0,0 y[33] = 0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[34] = 0,0,0,1,0,0,0,1,1,0 y[34] = 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[35] = 0,0,0,1,1,0,0,1,0,0 y[35] = 0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[36] = 0,0,1,0,0,1,0,0,0,0 y[36] = 0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[37] = 0,0,1,0,1,1,0,1,0,1 y[37] = 0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[38] = 0,0,1,1,0,1,0,0,1,0 y[38] = 0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[39] = 0,0,1,1,1,0,1,0,0,1 y[39] = 0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[40] = 0,1,0,0,0,1,1,0,0,1 y[40] = 0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[41] = 0,1,0,0,1,1,0,1,0,1 y[41] = 0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[42] = 0,1,0,1,0,1,0,1,0,1 y[42] = 0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[43] = 0,1,0,1,1,1,1,0,1,0 y[43] = 0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[44] = 0,1,1,0,0,0,0,0,1,0 y[44] = 0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[45] = 0,1,1,0,1,0,1,1,1,1 y[45] = 0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[46] = 0,1,1,1,0,1,0,1,1,1 y[46] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[47] = 0,1,1,1,1,0,0,0,0,1 y[47] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[48] = 1,0,0,0,0,1,0,1,0,1 y[48] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[49] = 1,0,0,0,1,1,1,0,1,1 y[49] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[50] = 1,0,0,1,0,1,0,1,1,0 y[50] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[51] = 1,0,0,1,1,0,0,1,1,0 y[51] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0
    x[52] = 1,0,1,0,0,0,1,0,0,1 y[52] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0
    x[53] = 1,0,1,0,1,1,0,1,0,0 y[53] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0
    x[54] = 1,0,1,1,0,1,0,0,0,1 y[54] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0
    x[55] = 1,0,1,1,1,0,0,0,0,0 y[55] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0
    x[56] = 1,1,0,0,0,0,1,1,1,0 y[56] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0
    x[57] = 1,1,0,0,1,1,1,1,0,1 y[57] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0
    x[58] = 1,1,0,1,0,0,1,1,0,0 y[58] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0
    x[59] = 1,1,0,1,1,1,0,1,1,1 y[59] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0
    x[60] = 1,1,1,0,0,0,1,0,0,0 y[60] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0
    x[61] = 1,1,1,0,1,0,1,0,1,1 y[61] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
    x[62] = 1,1,1,1,0,1,0,1,0,0 y[62] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0
    x[63] = 1,1,1,1,1,0,0,1,1,1 y[63] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
    x[64] = 0,0,0,0,0,0,1,1,0,1 y[64] = 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[65] = 0,0,0,0,1,0,1,0,0,0 y[65] = 0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[66] = 0,0,0,1,0,0,1,1,1,1 y[66] = 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[67] = 0,0,0,1,1,1,1,0,1,1 y[67] = 0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[68] = 0,0,1,0,0,0,1,0,0,1 y[68] = 0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[69] = 0,0,1,0,1,1,1,0,1,1 y[69] = 0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[70] = 0,0,1,1,0,0,1,0,0,1 y[70] = 0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[71] = 0,0,1,1,1,1,1,1,0,0 y[71] = 0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[72] = 0,1,0,0,0,1,0,0,1,1 y[72] = 0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[73] = 0,1,0,0,1,1,1,1,0,0 y[73] = 0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[74] = 0,1,0,1,0,1,1,0,1,1 y[74] = 0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[75] = 0,1,0,1,1,0,0,1,0,1 y[75] = 0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[76] = 0,1,1,0,0,0,1,0,1,1 y[76] = 0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[77] = 0,1,1,0,1,0,0,0,1,0 y[77] = 0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[78] = 0,1,1,1,0,0,0,0,1,1 y[78] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[79] = 0,1,1,1,1,1,0,1,0,1 y[79] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[80] = 1,0,0,0,0,1,1,0,0,0 y[80] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[81] = 1,0,0,0,1,0,1,1,0,0 y[81] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[82] = 1,0,0,1,0,0,1,1,1,1 y[82] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[83] = 1,0,0,1,1,0,1,0,0,1 y[83] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0
    x[84] = 1,0,1,0,0,1,0,0,0,0 y[84] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0
    x[85] = 1,0,1,0,1,0,1,0,1,0 y[85] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0
    x[86] = 1,0,1,1,0,1,0,0,1,1 y[86] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0
    x[87] = 1,0,1,1,1,0,0,1,1,1 y[87] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0
    x[88] = 1,1,0,0,0,1,0,0,0,1 y[88] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0
    x[89] = 1,1,0,0,1,0,0,1,1,1 y[89] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0
    x[90] = 1,1,0,1,0,0,0,1,0,1 y[90] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0
    x[91] = 1,1,0,1,1,1,1,1,1,0 y[91] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0
    x[92] = 1,1,1,0,0,0,1,0,0,1 y[92] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0
    x[93] = 1,1,1,0,1,1,1,0,1,1 y[93] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
    x[94] = 1,1,1,1,0,1,1,0,0,0 y[94] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0
    x[95] = 1,1,1,1,1,0,1,0,0,1 y[95] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
    x[96] = 0,0,0,0,0,0,1,1,1,1 y[96] = 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[97] = 0,0,0,0,1,0,1,0,0,1 y[97] = 0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[98] = 0,0,0,1,0,0,0,1,1,0 y[98] = 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x[99] = 0,0,0,1,1,0,0,0,0,0 y[99] = 0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0


## Split Into Training and Testing

We'll split the 100 rows in 90 rows of training, and 10 rows for testing.



```python
Percent = 0.10
x_train = []
y_train = []
x_test  = []
y_test  = []

# pick 10 indexes in 0-31.
indexes = [5, 7, 10, 20, 21, 29, 3, 11, 12, 25]

for i in range( len( x ) ):
    if i in indexes:
        x_test.append( x[i] )
        y_test.append( y[i] )
    else:
        x_train.append( x[i] )
        y_train.append( y[i] )

# display train and set xs and ys
for i in range( len( x_train ) ):
    print( "x_train[%2d] ="%i, ",".join( [str(k) for k in x_train[i] ] ), 
          "y_train[%2d] ="%i, ",".join( [str(k) for k in y_train[i] ] ) )

print()

for i in range( len( x_test ) ):
    print( "x_test[%2d] ="%i, ",".join( [str(k) for k in x_test[i] ] ), 
          "y_test[%2d] ="%i, ",".join( [str(k) for k in y_test[i] ] ) )
```

    x_train[ 0] = 0,0,0,0,0,1,0,0,0,1 y_train[ 0] = 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[ 1] = 0,0,0,0,1,1,1,0,1,1 y_train[ 1] = 0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[ 2] = 0,0,0,1,0,0,0,1,0,0 y_train[ 2] = 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[ 3] = 0,0,1,0,0,0,0,1,1,1 y_train[ 3] = 0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[ 4] = 0,0,1,1,0,1,1,0,1,1 y_train[ 4] = 0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[ 5] = 0,1,0,0,0,0,0,1,0,0 y_train[ 5] = 0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[ 6] = 0,1,0,0,1,0,0,0,1,1 y_train[ 6] = 0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[ 7] = 0,1,1,0,1,1,0,1,0,0 y_train[ 7] = 0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[ 8] = 0,1,1,1,0,0,1,0,1,0 y_train[ 8] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[ 9] = 0,1,1,1,1,1,1,1,0,1 y_train[ 9] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[10] = 1,0,0,0,0,0,1,1,0,1 y_train[10] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[11] = 1,0,0,0,1,1,1,1,0,1 y_train[11] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[12] = 1,0,0,1,0,0,0,1,0,1 y_train[12] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[13] = 1,0,0,1,1,1,0,0,0,1 y_train[13] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[14] = 1,0,1,1,0,1,0,1,0,0 y_train[14] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0
    x_train[15] = 1,0,1,1,1,1,0,0,1,1 y_train[15] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0
    x_train[16] = 1,1,0,0,0,1,0,0,1,0 y_train[16] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0
    x_train[17] = 1,1,0,1,0,0,1,1,1,0 y_train[17] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0
    x_train[18] = 1,1,0,1,1,0,1,0,0,1 y_train[18] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0
    x_train[19] = 1,1,1,0,0,1,0,1,0,0 y_train[19] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0
    x_train[20] = 1,1,1,1,0,1,0,1,0,0 y_train[20] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0
    x_train[21] = 1,1,1,1,1,1,0,1,1,0 y_train[21] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
    x_train[22] = 0,0,0,0,0,0,0,0,1,1 y_train[22] = 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[23] = 0,0,0,0,1,0,1,1,0,0 y_train[23] = 0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[24] = 0,0,0,1,0,0,0,1,1,0 y_train[24] = 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[25] = 0,0,0,1,1,0,0,1,0,0 y_train[25] = 0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[26] = 0,0,1,0,0,1,0,0,0,0 y_train[26] = 0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[27] = 0,0,1,0,1,1,0,1,0,1 y_train[27] = 0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[28] = 0,0,1,1,0,1,0,0,1,0 y_train[28] = 0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[29] = 0,0,1,1,1,0,1,0,0,1 y_train[29] = 0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[30] = 0,1,0,0,0,1,1,0,0,1 y_train[30] = 0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[31] = 0,1,0,0,1,1,0,1,0,1 y_train[31] = 0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[32] = 0,1,0,1,0,1,0,1,0,1 y_train[32] = 0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[33] = 0,1,0,1,1,1,1,0,1,0 y_train[33] = 0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[34] = 0,1,1,0,0,0,0,0,1,0 y_train[34] = 0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[35] = 0,1,1,0,1,0,1,1,1,1 y_train[35] = 0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[36] = 0,1,1,1,0,1,0,1,1,1 y_train[36] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[37] = 0,1,1,1,1,0,0,0,0,1 y_train[37] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[38] = 1,0,0,0,0,1,0,1,0,1 y_train[38] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[39] = 1,0,0,0,1,1,1,0,1,1 y_train[39] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[40] = 1,0,0,1,0,1,0,1,1,0 y_train[40] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[41] = 1,0,0,1,1,0,0,1,1,0 y_train[41] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[42] = 1,0,1,0,0,0,1,0,0,1 y_train[42] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0
    x_train[43] = 1,0,1,0,1,1,0,1,0,0 y_train[43] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0
    x_train[44] = 1,0,1,1,0,1,0,0,0,1 y_train[44] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0
    x_train[45] = 1,0,1,1,1,0,0,0,0,0 y_train[45] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0
    x_train[46] = 1,1,0,0,0,0,1,1,1,0 y_train[46] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0
    x_train[47] = 1,1,0,0,1,1,1,1,0,1 y_train[47] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0
    x_train[48] = 1,1,0,1,0,0,1,1,0,0 y_train[48] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0
    x_train[49] = 1,1,0,1,1,1,0,1,1,1 y_train[49] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0
    x_train[50] = 1,1,1,0,0,0,1,0,0,0 y_train[50] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0
    x_train[51] = 1,1,1,0,1,0,1,0,1,1 y_train[51] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
    x_train[52] = 1,1,1,1,0,1,0,1,0,0 y_train[52] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0
    x_train[53] = 1,1,1,1,1,0,0,1,1,1 y_train[53] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
    x_train[54] = 0,0,0,0,0,0,1,1,0,1 y_train[54] = 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[55] = 0,0,0,0,1,0,1,0,0,0 y_train[55] = 0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[56] = 0,0,0,1,0,0,1,1,1,1 y_train[56] = 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[57] = 0,0,0,1,1,1,1,0,1,1 y_train[57] = 0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[58] = 0,0,1,0,0,0,1,0,0,1 y_train[58] = 0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[59] = 0,0,1,0,1,1,1,0,1,1 y_train[59] = 0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[60] = 0,0,1,1,0,0,1,0,0,1 y_train[60] = 0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[61] = 0,0,1,1,1,1,1,1,0,0 y_train[61] = 0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[62] = 0,1,0,0,0,1,0,0,1,1 y_train[62] = 0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[63] = 0,1,0,0,1,1,1,1,0,0 y_train[63] = 0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[64] = 0,1,0,1,0,1,1,0,1,1 y_train[64] = 0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[65] = 0,1,0,1,1,0,0,1,0,1 y_train[65] = 0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[66] = 0,1,1,0,0,0,1,0,1,1 y_train[66] = 0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[67] = 0,1,1,0,1,0,0,0,1,0 y_train[67] = 0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[68] = 0,1,1,1,0,0,0,0,1,1 y_train[68] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[69] = 0,1,1,1,1,1,0,1,0,1 y_train[69] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[70] = 1,0,0,0,0,1,1,0,0,0 y_train[70] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[71] = 1,0,0,0,1,0,1,1,0,0 y_train[71] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[72] = 1,0,0,1,0,0,1,1,1,1 y_train[72] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[73] = 1,0,0,1,1,0,1,0,0,1 y_train[73] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[74] = 1,0,1,0,0,1,0,0,0,0 y_train[74] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0
    x_train[75] = 1,0,1,0,1,0,1,0,1,0 y_train[75] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0
    x_train[76] = 1,0,1,1,0,1,0,0,1,1 y_train[76] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0
    x_train[77] = 1,0,1,1,1,0,0,1,1,1 y_train[77] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0
    x_train[78] = 1,1,0,0,0,1,0,0,0,1 y_train[78] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0
    x_train[79] = 1,1,0,0,1,0,0,1,1,1 y_train[79] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0
    x_train[80] = 1,1,0,1,0,0,0,1,0,1 y_train[80] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0
    x_train[81] = 1,1,0,1,1,1,1,1,1,0 y_train[81] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0
    x_train[82] = 1,1,1,0,0,0,1,0,0,1 y_train[82] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0
    x_train[83] = 1,1,1,0,1,1,1,0,1,1 y_train[83] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
    x_train[84] = 1,1,1,1,0,1,1,0,0,0 y_train[84] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0
    x_train[85] = 1,1,1,1,1,0,1,0,0,1 y_train[85] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1
    x_train[86] = 0,0,0,0,0,0,1,1,1,1 y_train[86] = 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[87] = 0,0,0,0,1,0,1,0,0,1 y_train[87] = 0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[88] = 0,0,0,1,0,0,0,1,1,0 y_train[88] = 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_train[89] = 0,0,0,1,1,0,0,0,0,0 y_train[89] = 0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    
    x_test[ 0] = 0,0,0,1,1,1,1,1,0,0 y_test[ 0] = 0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_test[ 1] = 0,0,1,0,1,0,1,1,1,0 y_test[ 1] = 0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_test[ 2] = 0,0,1,1,1,0,0,1,1,0 y_test[ 2] = 0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_test[ 3] = 0,1,0,1,0,0,0,0,1,1 y_test[ 3] = 0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_test[ 4] = 0,1,0,1,1,0,1,0,1,0 y_test[ 4] = 0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_test[ 5] = 0,1,1,0,0,1,0,1,1,0 y_test[ 5] = 0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
    x_test[ 6] = 1,0,1,0,0,1,0,1,1,0 y_test[ 6] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0
    x_test[ 7] = 1,0,1,0,1,1,1,0,1,1 y_test[ 7] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0
    x_test[ 8] = 1,1,0,0,1,1,1,0,0,0 y_test[ 8] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0
    x_test[ 9] = 1,1,1,0,1,0,0,1,1,1 y_test[ 9] = 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0


## Package Xs and Ys as Numpy Arrays
We now make the train and test arrays into numpy arrays


```python
x_train_np = np.matrix( x_train ).astype( dtype=np.float32 )
y_train_np = np.matrix( y_train ).astype( dtype=np.float32 )
x_test_np = np.matrix( x_test ).astype( dtype=np.float32 )
y_test_np = np.matrix( y_test ).astype( dtype=np.float32 )

# get training size, number of features, and number of labels, using
# NN/ML vocabulary
train_size, num_features = x_train_np.shape
train_size, num_labels   = y_train_np.shape
    
# Get the number of epochs for training.
test_size, num_eval_features = x_test_np.shape
test_size, num_eval_labels   = y_test_np.shape

# Get the size of layer one.
if True:
    print( "tain size         = ", train_size )
    print( "num features      = ", num_features ) 
    print( "num labels        = ", num_labels )
    print()
    print( "test size         = ", test_size )
    print( "num eval features = ", num_eval_features ) 
    print( "num eval labels   = ", num_eval_labels )

```

    tain size         =  90
    num features      =  10
    num labels        =  32
    
    test size         =  10
    num eval features =  10
    num eval labels   =  32


# Definition of the Neural Network

Let's define the neural net.  We assume it has just 1 layer.

## Constants/Variables

We just have one, the learning rate with which the gradient optimizer will look for the optimal weights.  It's a factor used when following the gradient of the function y = W.x + b, in order to look for the minimum of the difference between y and the target.


```python
learning_Rate = 0.1
```

## Place-Holders

It will have place holders for
* the X input
* the Y target.  That's the vectors of Y values we generated above.  The network will generate its own version of y, which we'll compare to the target.  The closer the two are, the better.
* the drop-probability, which is defined as the "keep_probability", i.e. the probability a node from the neural net will be kept in the computation.  A value of 1.0 indicates that all the nodes are used in the processing of data through the network.


```python
x = tf.placeholder("float", shape=[None, num_features])
target = tf.placeholder("float", shape=[None, num_labels])
keep_prob = tf.placeholder(tf.float32) 
```

## Variables

The variables contain tensors that TensorFlow will manipulate.  Typically the Wi and bi coefficients of each layer.

We'll assume just one later for right now, with num_features inputs (the width of the X vectors), and num_labels outputs (the width of the Y vectors).  We initialize W0 and b0 with random values taken from a normal distribution.



```python
W0 = tf.Variable( tf.random_normal( [num_features, num_labels ] ) )
b0 = tf.Variable( tf.random_normal( [num_labels] ) )
W1 = tf.Variable( tf.random_normal( [num_labels, num_labels * 2 ] ) )
b1 = tf.Variable( tf.random_normal( [num_labels * 2] ) )
W2 = tf.Variable( tf.random_normal( [num_labels * 2, num_labels ] ) )
b2 = tf.Variable( tf.random_normal( [num_labels] ) )

```

## Model

The model simply defines what the output of the NN, y, is as a function of the input x.  The *softmax* function transforms the output into probabilities between 0 and 1.  This is what we need since we want the output of our network to match the 1-hot vector which is the format the y vectors are coded in.



```python
#y0 = tf.nn.sigmoid( tf.matmul(x, W0) + b0 ) 
y0 = tf.nn.sigmoid( tf.matmul(x, W0) + b0 ) 
y1 = tf.nn.sigmoid( tf.matmul(y0, W1) + b1 )
y =  tf.matmul( y1, W2) + b2
#y = tf.nn.softmax( tf.matmul( y0, W1) + b1 )
```


## Training

We now define the cost operation, **cost_op**, i.e. measuring how "bad" the output of the network is compared to the correct output.




```python
#prediction = tf.reduce_sum( tf.mul( tf.nn.softmax( y ), target ), reduction_indices=1 )
#accuracy = tf.reduce_mean ( prediction )
#cost_op = tf.reduce_mean( tf.sub( 1.0, tf.reduce_sum( tf.mul( y, target ), reduction_indices=1 ) ) )
          
#cost_op =  tf.reduce_mean( 
#              tf.sub( 1.0, tf.reduce_sum( tf.mul( target, tf.nn.softmax(y) ), reduction_indices=[1] ) ) 
#           )

# The cost_op below yields an ccuracy on training data of 0.86% and an accuracy on test data = 0.49% 
# for 1000 epochs and a batch size of 10.
cost_op = tf.reduce_mean(
      tf.nn.softmax_cross_entropy_with_logits( labels = target, logits = y ) )
```

And now the training operation, or **train_op**, which is given the **cost_op**


```python
#train_op = tf.train.GradientDescentOptimizer( learning_rate = learning_Rate ).minimize( cost_op )
train_op = tf.train.AdagradOptimizer( learning_rate = learning_Rate ).minimize( cost_op )
```

# Initialization Phase

We need to create an *initialization operation*, init_op, as well.  It won't be executed yet, not until the session starts, but we have to do it first.



```python
init_op = tf.initialize_all_variables()

```

# Start the Session

We are now ready to start a session!



```python
sess = tf.Session()
sess.run( init_op )
```

## Training the NN

We now train the Neural Net for 1000 epoch.  In each epoch we feed just one vector of x to the network.



```python
batchSize = 5

prediction = tf.equal( tf.argmax( y, 1 ), tf.argmax( target, 1) )
accuracy = tf.reduce_mean ( tf.cast( prediction, tf.float32 ) )

for epoch in range( 10000 ):
    for i in range( 0, train_size, batchSize ):
        xx = x_train_np[ i:i+batchSize, : ]
        yy = y_train_np[ i:i+batchSize, : ]
        sess.run( train_op, feed_dict={x: xx, target: yy} )
        
    
    if epoch%100 == 0:
        co, to = sess.run( [cost_op,train_op], feed_dict={x: x_train_np, target: y_train_np} )
        print( epoch, "cost =", co, end=" " )
        accuracyNum = sess.run( accuracy, feed_dict={x: x_train_np, target : y_train_np} )
        print( "Accuracy on training data = %1.2f%%" % (accuracyNum*100), end = " " )
        accuracyNum = sess.run( accuracy, feed_dict={ x: x_test_np, target : y_test_np} )
        print( "Accuracy on test data = %1.2f%%" % ( accuracyNum*100 ) )

if False:
    print( "y = ", sess.run( y, feed_dict={ x: x_train_np, target : y_train_np} ) )
    print( "softmax(y) = ", sess.run( tf.nn.softmax( y ), feed_dict={ x: x_train_np, target : y_train_np} ) )
    print( "tf.mul(tf.nn.softmax(y), target) = ",
                         sess.run( tf.mul( tf.nn.softmax( y ), target ), 
                                   feed_dict={ x: x_train_np, target : y_train_np} ) )

#  
#prediction = tf.reduce_sum( tf.mul( tf.nn.softmax( y ), target ), reduction_indices=1 )
accuracyNum = sess.run( accuracy, feed_dict={x: x_train_np, target : y_train_np} )
print( "Final Accuracy on training data = %1.2f%%" % (100.0*accuracyNum) )

accuracyNum = sess.run( accuracy, feed_dict={ x: x_test_np, target : y_test_np} )
print( "Final Accuracy on test data = %1.2f%%" % (100.0*accuracyNum) )

```

    0 cost = 3.61614 Accuracy on training data = 14.44% Accuracy on test data = 10.00%
    100 cost = 0.0624902 Accuracy on training data = 100.00% Accuracy on test data = 30.00%
    200 cost = 0.0269957 Accuracy on training data = 100.00% Accuracy on test data = 50.00%
    300 cost = 0.0169137 Accuracy on training data = 100.00% Accuracy on test data = 50.00%
    400 cost = 0.0122166 Accuracy on training data = 100.00% Accuracy on test data = 50.00%
    500 cost = 0.00951857 Accuracy on training data = 100.00% Accuracy on test data = 60.00%
    600 cost = 0.00777421 Accuracy on training data = 100.00% Accuracy on test data = 70.00%
    700 cost = 0.00655674 Accuracy on training data = 100.00% Accuracy on test data = 70.00%
    800 cost = 0.00566033 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    900 cost = 0.00497359 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    1000 cost = 0.00443122 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    1100 cost = 0.00399232 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    1200 cost = 0.00363012 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    1300 cost = 0.00332625 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    1400 cost = 0.00306783 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    1500 cost = 0.00284544 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    1600 cost = 0.00265214 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    1700 cost = 0.00248257 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    1800 cost = 0.00233273 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    1900 cost = 0.00219929 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    2000 cost = 0.00207979 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    2100 cost = 0.00197217 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    2200 cost = 0.00187475 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    2300 cost = 0.00178615 Accuracy on training data = 100.00% Accuracy on test data = 80.00%
    2400 cost = 0.00170524 Accuracy on training data = 100.00% Accuracy on test data = 80.00%



```python

```


```python

```
